<?php

namespace Drupal\media_folder_management\Controller;

use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\workflows\Entity\Workflow;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatterInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Render\Markup;
use Drupal\Component\Utility\SortArray;
use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\Unicode;
use Drupal\Core\Breadcrumb\Breadcrumb;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Routing\RedirectDestinationInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileExists;
use Drupal\Core\File\FileSystem;
use Drupal\Core\File\FileSystemInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\File\FileUrlGenerator;
use Drupal\Core\Utility\Token;
use Drupal\media_folder_management\Entity\Access\MediaFolderFileAccessCheck;
use Drupal\file\FileRepository;
use Drupal\file\Validation\FileValidator;
use Drupal\file\FileUsage\DatabaseFileUsageBackend;

/**
 * Class File media folder file Controller.
 */
class MediaFolderFileController extends ControllerBase {

  /**
   * The cache backend service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheBackend;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityManager;

  /**
   * Current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The redirect destination service.
   *
   * @var \Drupal\Core\Routing\RedirectDestinationInterface
   */
  protected $redirectDestination;

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Drupal\Core\Render\RendererInterface definition.
   *
   * @var Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Form builder will be used via Dependency Injection.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * File url generator object.
   *
   * @var \Drupal\Core\File\FileUrlGenerator
   */
  protected $fileUrlGenerator;

  /**
   * File file repository object.
   *
   * @var \Drupal\file\FileRepository
   */
  protected $fileRepository;

  /**
   * File file validator object.
   *
   * @var \Drupal\file\Validation\FileValidator
   */
  protected $fileValidator;

  /**
   * File file system object.
   *
   * @var \Drupal\Core\File\FileSystem
   */
  protected $fileSystem;

  /**
   * File token object.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * File file usage object.
   *
   * @var \Drupal\file\FileUsage\DatabaseFileUsageBackend
   */
  protected $fileUsage;

  /**
   * The term storage handler.
   *
   * @var \Drupal\taxonomy\TermStorageInterface
   */
  protected $storageController;

  /**
   * The media folder file settings.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $mediaFolderFileSettings;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('date.formatter'),
      $container->get('media_folder_management.media_folder_file_cache'),
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('redirect.destination'),
      $container->get('config.factory'),
      $container->get('request_stack'),
      $container->get('renderer'),
      $container->get('form_builder'),
      $container->get('file_url_generator'),
      $container->get('file.repository'),
      $container->get('file.validator'),
      $container->get('file_system'),
      $container->get('token'),
      $container->get('file.usage')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function __construct(
    DateFormatterInterface $date_formatter,
    CacheBackendInterface $cache_backend,
    EntityTypeManagerInterface $entity_manager,
    AccountInterface $current_user,
    RedirectDestinationInterface $redirect_destination,
    ConfigFactoryInterface $config_factory,
    RequestStack $request_stack,
    RendererInterface $renderer,
    FormBuilderInterface $form_builder,
    FileUrlGenerator $fileUrlGenerator,
    FileRepository $file_repository,
    FileValidator $file_validator,
    FileSystem $file_system,
    Token $token,
    DatabaseFileUsageBackend $file_usage,
  ) {
    $this->dateFormatter = $date_formatter;
    $this->cacheBackend = $cache_backend;
    $this->entityManager = $entity_manager;
    $this->currentUser = $entity_manager->getStorage('user')->load($current_user->id());
    $this->redirectDestination = $redirect_destination;
    $this->requestStack = $request_stack;
    $this->renderer = $renderer;
    $this->formBuilder = $form_builder;
    $this->fileUrlGenerator = $fileUrlGenerator;
    $this->fileRepository = $file_repository;
    $this->fileValidator = $file_validator;
    $this->fileSystem = $file_system;
    $this->token = $token;
    $this->fileUsage = $file_usage;
    $this->storageController = $entity_manager->getStorage('taxonomy_term');
    $this->mediaFolderFileSettings = $config_factory->get('media_folder_management.settings');
  }

  /**
   * Get user folders.
   */
  public static function geAllUserFoldersArray($storageController, $with_root = FALSE) {
    $folder_array = ($with_root) ? ['root' => t('Root')] : [];
    $folders = $storageController->loadTree('media_folder', 0, NULL, FALSE);
    foreach ($folders as $folder) {
      $depth = ($with_root) ? ($folder->depth + 1) : $folder->depth;
      $folder_array[$folder->tid] = str_repeat('-', $depth) . ' ' . $folder->name;
    }

    return $folder_array;
  }

  /**
   * Get tree Nested Array.
   */
  public function treeNestedArray(&$build, $tree, $my_folders, $folder_id) {
    foreach ($tree as $term) {
      if (!in_array($term->id(), $my_folders)) {
        continue;
      }
      $key = strtolower($term->getName()) . '-' . $term->id();
      $link = Link::fromTextAndUrl($term->getName(), Url::fromRoute('media_folder_management.folder', ['folder' => $term->id()]))->toRenderable();
      $link['#attributes']['data-id'] = $term->id();
      $build[$key] = [
        'id' => $term->id(),
        'link' => $link,
        'class' => ($folder_id == $term->id()) ? 'active' : '',
        'children' => [],
      ];
      $children = $this->storageController->getChildren($term);
      if (!empty($children)) {
        $this->treeNestedArray($build[$key]['children'], $children, $my_folders, $folder_id);
      }
    }
    ksort($build);
  }

  /**
   * Get tree Nested Array.
   */
  public function treeNestedArrayActiveTrail(&$build, $folder_id, $paths = FALSE) {
    if ($paths === FALSE) {
      $paths = $this->treeNestedArrayActiveTrailPaths($build, $folder_id);
    }

    if (!empty($paths)) {
      foreach ($paths as $path) {
        if (!empty($build[$path])) {
          $build[$path]['class'] .= ' active-trail';
          $this->treeNestedArrayActiveTrail($build[$path]['children'], $folder_id, $paths);
        }
      }
    }
  }

  /**
   * Get tree Nested Array.
   */
  public function treeNestedArrayActiveTrailPaths($build, $folder_id) {
    $paths = [];
    $iterator = new \RecursiveIteratorIterator(new \RecursiveArrayIterator($build), \RecursiveIteratorIterator::SELF_FIRST);
    foreach ($iterator as $v) {
      if (is_array($v) && !empty($v['id']) && $v['id'] == $folder_id) {
        for ($p = [], $i = 0, $z = $iterator->getDepth(); $i <= $z; $i++) {
          $p = $iterator->getSubIterator($i)->key();
          if ($p != 'children') {
            $paths[] = $p;
          }
        }
      }
    }

    return $paths;
  }

  /**
   * Get user folders.
   */
  public function geAllUserFoldersTree($folder_id) {
    $folders = $this->storageController->loadTree('media_folder', 0, 1, TRUE);
    $my_folders = MediaFolderFileAccessCheck::getUserViewFoldersAccess(0, $this->currentUser(), $this->storageController);
    $build = [];
    $this->treeNestedArray($build, $folders, $my_folders, $folder_id);
    $this->treeNestedArrayActiveTrail($build, $folder_id);

    return $build;
  }

  /**
   * Get sub folders.
   */
  public static function geAllSubFolders($folder, $storageController) {
    $tid = ($folder == 'all' || $folder == 'root') ? 0 : $folder;
    $folder_array = [];
    $folders = $storageController->loadTree('media_folder', $tid, NULL, FALSE);
    foreach ($folders as $folder) {
      $folder_array[$folder->tid] = $folder->tid;
    }

    return $folder_array;
  }

  /**
   * Get title.
   */
  public function getTitle(mixed $folder = NULL) {
    if (!$folder) {
      return $this->mediaFolderFileSettings->get('page_name');
    }
    $element = ($folder && ($folder->bundle() == 'media_folder')) ? $folder->getName() : '';

    return $this->mediaFolderFileSettings->get('page_name') . ' | ' . $element;
  }

  /**
   * Add file title.
   */
  public function getAddFileTitle($folder) {
    $parent = $this->storageController->load($folder);
    return $this->t('Add file to') . ' ' . $parent->getName();
  }

  /**
   * Get Folder BreadCrumb.
   */
  public static function getFolderBreadCrumb($folder, Breadcrumb &$breadcrumb, $storageController) {
    $ancestors = $storageController->loadAllParents($folder);
    if (!empty($ancestors)) {
      $ancestors = array_reverse($ancestors);
      $folder = array_pop($ancestors);
      foreach ($ancestors as $ancestor) {
        $breadcrumb->addLink(Link::fromTextAndUrl($ancestor->getName(), Url::fromRoute('media_folder_management.folder', ['folder' => $ancestor->id()])));
      }
      $breadcrumb->addLink(Link::createFromRoute($folder->getName(), '<nolink>'));
    }
  }

  /**
   * Get media folder file Workflow.
   */
  public static function getMediaFolderFileWorkflow() {
    if (\Drupal::service('module_handler')->moduleExists('content_moderation')) {
      $workflows = Workflow::loadMultipleByType('content_moderation');
      foreach ($workflows as $workflow) {
        $type_settings = $workflow->get('type_settings');
        if (array_key_exists('media_folder_file', $type_settings['entity_types'])) {
          return $workflow;
        }
      }
    }
    return FALSE;
  }

  /**
   * Get folder actions.
   */
  private function getFolderActions($folder) {
    $action_links = [];

    $destination = $this->redirectDestination->getAsArray();
    if (!empty($destination['destination'])) {
      $url = parse_url($destination['destination']);
      $destination['destination'] = str_replace('/ajax', '', $url['path']);
    }

    if (!$folder && MediaFolderFileAccessCheck::hasAdminFolderPermission($this->currentUser)) {
      $action_links['new'] = [
        'title' => $this->t('New folder'),
        'url' => Url::fromRoute('media_folder_management.add_root_folder', [], ['query' => $destination]),
        'class' => ['new-folder'],
      ];
      $this->transformDialogLink($action_links['new'], $this->t('New folder'), TRUE, '90%');
    }
    elseif ($folder && (MediaFolderFileAccessCheck::hasAdminFolderPermission($this->currentUser) || MediaFolderFileAccessCheck::hasCreateFolderPermission($folder, $this->currentUser, TRUE))) {
      $action_links['new'] = [
        'title' => $this->t('New folder'),
        'url' => Url::fromRoute('media_folder_management.add_folder', ['folder' => $folder->id()], ['query' => $destination]),
        'class' => ['new-folder'],
      ];
      $this->transformDialogLink($action_links['new'], $this->t('New folder'), TRUE, '90%');
    }

    if ($folder) {
      if (MediaFolderFileAccessCheck::hasFolderPermission('create', $folder, $this->currentUser)) {
        $action_links['upload'] = [
          'title' => $this->t('Upload file'),
          'url' => Url::fromRoute('entity.media_folder_file.add_form', [
            'entity_type_id' => 'media_folder_file',
            'bundle_parameter' => 'media_folder_file_type',
            'media_folder_file_type' => 'file',
            'folder' => $folder->id(),
          ],
          ['query' => $destination]),
          'class' => ['upload-file'],
        ];
        $this->transformDialogLink($action_links['upload'], $this->t('Upload file'), TRUE, '70%');
      }

      if (MediaFolderFileAccessCheck::hasEditFolderPermission($folder, $this->currentUser, TRUE)) {
        $action_links['edit'] = [
          'title' => $this->t('Edit folder'),
          'url' => Url::fromRoute('media_folder_management.edit_folder', ['folder' => $folder->id()], ['query' => $destination]),
          'class' => ['edit-action'],
        ];
        $this->transformDialogLink($action_links['edit'], $this->t('Edit folder'), TRUE, '70%');
      }

      if (MediaFolderFileAccessCheck::hasDeleteFolderPermission($folder, $this->currentUser, TRUE)) {
        $action_links['delete'] = [
          'title' => $this->t('Delete folder'),
          'url' => Url::fromRoute('media_folder_management.delete_folder', ['folder' => $folder->id()]),
          'class' => ['delete-action'],
        ];
        $this->transformDialogLink($action_links['delete'], $this->t('Delete folder'), TRUE, '70%');
      }

      if (MediaFolderFileAccessCheck::hasViewFolderPermission($folder, $this->currentUser, TRUE)) {
        $action_links['share'] = [
          'title' => $this->t('Share link'),
          'url' => Url::fromRoute('media_folder_management.share', ['folder' => $folder->id()], ['query' => $destination]),
          'class' => ['share-action'],
        ];
        $this->transformDialogLink($action_links['share'], $this->t('Share link'), TRUE, '70%');
      }

      if (MediaFolderFileAccessCheck::hasAdminFolderPermission($this->currentUser)) {
        $action_links['perms'] = [
          'title' => $this->t('Permissions'),
          'url' => Url::fromRoute('media_folder_management.folder_perms', ['folder' => $folder->id()], ['query' => $destination]),
          'class' => ['perms-action'],
        ];
        $this->transformDialogLink($action_links['perms'], $this->t('Permissions'), TRUE, '70%');
      }
    }
    elseif (MediaFolderFileAccessCheck::hasAdminFolderPermission($this->currentUser)) {
      $action_links['manage'] = [
        'title' => $this->t('Manage folders'),
        'url' => Url::fromRoute('entity.taxonomy_vocabulary.overview_form', ['taxonomy_vocabulary' => 'media_folder'], ['query' => $destination]),
        'class' => ['manage-folder'],
      ];
      $this->transformDialogLink($action_links['manage'], $this->t('Manage folders'), TRUE);
    }

    if (!empty($action_links)) {
      $actions = [
        '#type' => 'dropbutton',
        '#links' => $action_links,
      ];

      return $actions;
    }

    return '';
  }

  /**
   * Get File Classes.
   */
  public static function getFileClasses($file, $file_field) {
    $classes = [
      'folder-icon',
      'folder-icon-file',
    ];

    if ($file_field->field_locked->value) {
      $classes[] = 'locked';
    }

    switch ($file->getMimeType()) {
      case 'text/plain':
        $classes[] = 'folder-icon-file-code';
        break;

      case 'application/pdf':
        $classes[] = 'folder-icon-file-pdf';
        break;

      case 'application/msword':
      case 'application/vnd.openxmlformats-officedocument.wordprocessingml.document':
      case 'application/vnd.oasis.opendocument.text':
        $classes[] = 'folder-icon-file-word';
        break;

      case 'application/vnd.oasis.opendocument.spreadsheet':
      case 'application/vnd.ms-excel':
      case 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet':
        $classes[] = 'folder-icon-file-excel';
        break;

      case 'application/vnd.ms-powerpoint':
      case 'application/vnd.openxmlformats-officedocument.presentationml.presentation':
      case 'application/vnd.oasis.opendocument.presentation':
        $classes[] = 'folder-icon-file-powerpoint';
        break;

      case 'application/zip':
      case 'application/octet-stream':
      case 'application/x-zip-compressed':
      case 'multipart/x-zip':
      case 'application/x-rar-compressed':
      case 'application/vnd.rar':
      case 'application/zip':
        $classes[] = 'folder-icon-file-zip';
        break;

      case 'image/png':
      case 'image/gif':
      case 'image/jpeg':
      case 'image/jpg':
      case 'image/bmp':
        $classes[] = 'folder-icon-file-image';
        break;

      default:
        $classes[] = 'folder-icon-doc';
        break;
    }

    return implode(' ', $classes);
  }

  /**
   * Get human Filesize.
   */
  private function humanFilesize($bytes, $decimals = 2) {
    $size = ['B', 'kB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
    $factor = floor((strlen($bytes) - 1) / 3);

    return (!empty($size[$factor])) ? sprintf("%.{$decimals}f", $bytes / pow(1024, $factor)) . $size[$factor] : '';
  }

  /**
   * Get cookie.
   */
  private function getCookie($name, $default) {
    $cookies = $this->requestStack->getCurrentRequest()->cookies;
    if ($cookies->has($name)) {
      return $cookies->get($name);
    }

    return $default;
  }

  /**
   * Create Link.
   */
  private function createLink($text, $url, $classes = [], $title = FALSE, $query = FALSE) {
    $link = Link::fromTextAndUrl($text, $url)->toRenderable();
    $link['#attributes'] = [
      'class' => $classes,
      'title' => ($title ? $title : $text),
    ];
    if ($query) {
      $destination = $this->redirectDestination->getAsArray();
      if (!empty($destination['destination'])) {
        $url = parse_url($destination['destination']);
        $destination['destination'] = str_replace('/ajax', '', $url['path']);
      }
      $link['#url']->setOption('query', $destination);
    }

    return $link;
  }

  /**
   * Create Dialog Link.
   */
  private function createDialogLink($text, $url, $classes = [], $title = FALSE, $query = FALSE, $width = '40%') {
    $link = $this->createLink($text, $url, $classes, $title, $query);
    $link['#attributes']['class'][] = 'use-ajax';
    $link['#attributes']['data-dialog-type'] = 'modal';
    $link['#attributes']['data-dialog-options'] = Json::encode(['width' => $width]);
    $link['#attached'] = [
      'library' => [
        'core/drupal.dialog.ajax',
      ],
    ];
    return $link;
  }

  /**
   * Transform Dialog Link.
   */
  private function transformDialogLink(&$link, $title = FALSE, $query = FALSE, $width = '40%') {
    $link['attributes']['class'][] = 'use-ajax';
    $link['attributes']['data-dialog-type'] = 'modal';
    $link['attributes']['data-dialog-options'] = Json::encode(['width' => $width]);
    $link['#attached'] = [
      'library' => [
        'core/drupal.dialog.ajax',
      ],
    ];
  }

  /**
   * Get Buttons.
   */
  private function getButtons() {
    $buttons = $render_buttons = [];

    $buttons['thumbs'] = $this->createLink('', Url::fromRoute('media_folder_management.cookies', [
      'name' => 'view-type',
      'value' => 'thumbs',
    ]), ['thumbs'], $this->t('Folder view'), TRUE);
    $buttons['list'] = $this->createLink('', Url::fromRoute('media_folder_management.cookies', [
      'name' => 'view-type',
      'value' => 'list',
    ]), ['list'], $this->t('List view'), TRUE);
    $view = $this->getCookie('view-type', $this->mediaFolderFileSettings->get('default_view'));
    $buttons[$view]['#attributes']['class'][] = 'active';

    $buttons['az'] = $this->createLink('', Url::fromRoute('media_folder_management.cookies', [
      'name' => 'view-order',
      'value' => 'az',
    ]), ['az'], $this->t('Alphabetic ascending'), TRUE);
    $buttons['za'] = $this->createLink('', Url::fromRoute('media_folder_management.cookies', [
      'name' => 'view-order',
      'value' => 'za',
    ]), ['za'], $this->t('Alphabetic descending'), TRUE);
    $buttons['date-asc'] = $this->createLink('', Url::fromRoute('media_folder_management.cookies', [
      'name' => 'view-order',
      'value' => 'date-asc',
    ]), ['date-asc'], $this->t('Date ascending'), TRUE);
    $buttons['date-desc'] = $this->createLink('', Url::fromRoute('media_folder_management.cookies', [
      'name' => 'view-order',
      'value' => 'date-desc',
    ]), ['date-desc'], $this->t('Date descending'), TRUE);
    $order = $this->getCookie('view-order', $this->mediaFolderFileSettings->get('default_order'));
    $buttons[$order]['#attributes']['class'][] = 'active';

    foreach ($buttons as $button) {
      $render_buttons[] = $this->renderer->render($button);
    }

    return implode('', $render_buttons);
  }

  /**
   * Get Form.
   */
  private function getForm() {
    $form = $this->formBuilder->getForm('\Drupal\media_folder_management\Form\MediaFolderFileSearchForm');

    return $this->renderer->render($form);
  }

  /**
   * Get Item Actions.
   */
  private function getItemActions(mixed $folder = FALSE, mixed $file = FALSE) {
    $items = $render_items = [];

    if ($folder) {
      if (MediaFolderFileAccessCheck::hasEditFolderPermission($folder, $this->currentUser, TRUE)) {
        $items['edit'] = $this->createDialogLink($this->t('Edit folder'), Url::fromRoute('media_folder_management.edit_folder', ['folder' => $folder->id()]), ['edit-action'], $this->t('Edit'), TRUE, '90%');
      }

      if (MediaFolderFileAccessCheck::hasDeleteFolderPermission($folder, $this->currentUser, TRUE)) {
        $items['delete'] = $this->createDialogLink($this->t('Delete folder'), Url::fromRoute('media_folder_management.delete_folder', ['folder' => $folder->id()]), ['delete-action'], $this->t('Delete'), FALSE, '90%');
      }

      if (MediaFolderFileAccessCheck::hasAdminFolderPermission($this->currentUser)) {
        $items['perms'] = $this->createDialogLink($this->t('Folder permissions'), Url::fromRoute('media_folder_management.folder_perms', ['folder' => $folder->id()]), ['perms-action'], $this->t('Permissions'), TRUE, '90%');
      }

      if (MediaFolderFileAccessCheck::hasViewFolderPermission($folder, $this->currentUser, TRUE)) {
        $items['share'] = $this->createDialogLink($this->t('Share link'), Url::fromRoute('media_folder_management.share', ['folder' => $folder->id()]), ['share-action'], $this->t('Share link'), TRUE, '90%');
      }
    }
    elseif ($file) {
      if (MediaFolderFileAccessCheck::hasLockFileAccess($file, $this->currentUser, $this->storageController)) {
        $text = ($file->field_locked->value) ? $this->t('Unlock') : $this->t('Lock');
        $class = ($file->field_locked->value) ? ['unlock-action'] : ['lock-action'];
        $items['lock'] = $this->createDialogLink($text, Url::fromRoute('entity.media_folder_file.lock', ['media_folder_file' => $file->id()]), $class, $text, TRUE);
      }
      if (MediaFolderFileAccessCheck::hasEditFileAccess($file, $this->currentUser, $this->storageController)) {
        $items['edit'] = $this->createDialogLink($this->t('Edit file'), Url::fromRoute('entity.media_folder_file.edit_form', ['media_folder_file' => $file->id()]), ['edit-action'], $this->t('Edit'), TRUE);
      }
      if (MediaFolderFileAccessCheck::hasFileRevisionAccess($file, $this->currentUser, $this->storageController)) {
        $items['revision'] = $this->createDialogLink($this->t('File revisions'), Url::fromRoute('entity.media_folder_file.version_history', ['media_folder_file' => $file->id()]), ['revisions-action'], $this->t('RevisionsEdit'), TRUE, '70%');
      }
      if (MediaFolderFileAccessCheck::hasDeleteFileAccess($file, $this->currentUser, $this->storageController)) {
        $items['delete'] = $this->createDialogLink($this->t('Delete file'), Url::fromRoute('entity.media_folder_file.delete_form', ['media_folder_file' => $file->id()]), ['delete-action'], $this->t('Delete'), TRUE);
      }
    }

    foreach ($items as $item) {
      $render_items[] = $this->renderer->render($item);
    }

    return implode('', $render_items);
  }

  /**
   * Get File Count.
   */
  private function getFileCount($folder) {
    $sub_folder_count = $this->storageController->loadChildren($folder);
    $query = $this->entityManager->getStorage('media_folder_file')->getQuery();
    $query->condition('bundle', 'file');
    $query->condition('field_folder', $folder);
    $query->condition('status', 1);
    $entity_ids = $query->accessCheck(FALSE)->execute();

    return count($sub_folder_count) + count($entity_ids);
  }

  /**
   * Get Locked Info.
   */
  private function getLockedInfo(mixed $file = FALSE) {
    $items = [];

    if ($file && $file->field_locked->value) {
      $items[] = ' data-locked-by="' . $file->getRevisionLogMessage() . '"';
    }

    return implode('', $items);
  }

  /**
   * Order items.
   */
  private function itemsOrder($items) {
    $view_order = $this->getCookie('view-order', $this->mediaFolderFileSettings->get('default_order'));

    if ($view_order == 'za' || $view_order == 'az') {
      uasort($items, [SortArray::class, 'sortByTitleProperty']);
    }
    elseif ($view_order == 'date-asc' || $view_order == 'date-desc') {
      uasort($items, [SortArray::class, 'sortByWeightProperty']);
    }

    if ($view_order == 'za' || $view_order == 'date-asc') {
      return array_reverse($items);
    }

    return $items;
  }

  /**
   * Get Folder Entities.
   */
  private function getFolderEntities($folder) {
    $query = $this->entityManager->getStorage('media_folder_file')->getQuery();
    $query->condition('bundle', 'file');
    $query->condition('field_folder', $folder->id());
    $query->condition('status', 1);
    $entity_ids = $query->accessCheck(FALSE)->execute();

    if (!empty($entity_ids)) {
      return $this->entityManager->getStorage('media_folder_file')->loadMultiple($entity_ids);
    }

    return [];
  }

  /**
   * Get Folder Files.
   */
  private function getFolderFiles($folder, &$folders) {
    if ($folder) {
      $files = [];
      $view_type = $this->getCookie('view-type', $this->mediaFolderFileSettings->get('default_view'));

      $values = $this->getFolderEntities($folder);
      foreach ($values as $file_entity) {
        if (!empty($file_entity->field_file->entity)) {
          $file = $file_entity->field_file->entity;
          $ext = pathinfo($file->getFilename(), PATHINFO_EXTENSION);
          $type = $this->humanFilesize($file->getSize());

          $files[] = [
            '#theme' => 'media_folder_folder',
            '#title' => Unicode::truncate($file_entity->getName(), 30, FALSE, TRUE),
            '#weight' => $file_entity->field_file->entity->changed->value,
            '#desc' => $file_entity->field_file->description,
            '#link' => $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri()),
            '#classes' => $this->getFileClasses($file, $file_entity),
            '#wrapper_classes' => 'folder-file',
            '#attr' => Markup::create('download data-ext=".' . $ext . '" data-size="' . $type . '"' . $this->getLockedInfo($file_entity)),
            '#size' => ($view_type == 'list') ? $type : '',
            '#ftype' => ($view_type == 'list') ? $ext : '',
            '#actions' => Markup::create($this->getItemActions(FALSE, $file_entity)),
            '#fid' => $file_entity->id(),
            '#cache' => [
              'tags' => [
                'media_folder_item:' . $file_entity->id(),
              ],
              'contexts' => [
                'user',
              ],
            ],
          ];
        }
      }
      $files_ordered = $this->itemsOrder($files);

      $folders = array_merge($folders, $files_ordered);
    }
  }

  /**
   * Get Up Button.
   */
  public function getUpButton($parent) {
    if (isset($parent->parent->target_id) && $parent->parent->target_id == 0) {
      $link = Link::fromTextAndUrl('', Url::fromRoute('media_folder_management.file_explorer'))->toRenderable();
      $link['#attributes']['title'] = $this->t('Parent folder');
      $link['#attributes']['data-label'] = $this->t('Parent folder');

      return $link;
    }
    elseif (isset($parent->parent->target_id)) {
      $link = Link::fromTextAndUrl('', Url::fromRoute('media_folder_management.folder', ['folder' => $parent->parent->target_id]))->toRenderable();
      $link['#attributes']['title'] = $this->t('Parent folder');
      $link['#attributes']['data-label'] = $this->t('Parent folder');

      return $link;
    }

    return Markup::create('<span></span>');
  }

  /**
   * Get Breadcrumb.
   */
  public function getBreadcrumb($parent) {
    if (isset($parent->parent->target_id)) {
      $parents = [];
      $term = clone $parent;
      $parents[$parent->id()] = '<span>' . Unicode::truncate($parent->getName(), 15, FALSE, TRUE) . '</span>';
      while (!empty($term->parent->target_id)) {
        $term = $term->parent->entity;
        $parents[$term->id()] = Link::fromTextAndUrl(Unicode::truncate($term->getName(), 15, FALSE, TRUE), Url::fromRoute('media_folder_management.folder', ['folder' => $term->id()]))->toString();
      }
      $parents[0] = Link::fromTextAndUrl($this->t('Root'), Url::fromRoute('media_folder_management.file_explorer'))->toString();
      $parents = array_reverse($parents);

      return Markup::create(implode('<i></i>', $parents));
    }

    return Markup::create('<span class="root">' . $this->t('Root') . '</span>');
  }

  /**
   * Get Raw actions.
   */
  public function getRawActions($actions) {
    $actions_raw = [];
    if (!empty($actions['#links'])) {
      foreach ($actions['#links'] as $key => $link) {
        $actions_raw[$key] = [
          '#type'  => 'link',
          '#title' => $link['title'],
          '#url'  => $link['url'],
          '#attributes' => $link['attributes'],
          '#attached' => $link['#attached'],
        ];
        $actions_raw[$key]['#attributes']['class'][] = $link['class'][0];
      }
    }

    return $actions_raw;
  }

  /**
   * Display media folder.
   */
  public function fileExplorer($folder = NULL) {
    if ($folder && ($folder->bundle() !== 'media_folder')) {
      throw new NotFoundHttpException();
    }

    $folder_id = ($folder) ? $folder->id() : 0;
    $folders = [];
    $tree = $this->storageController->loadTree('media_folder', $folder_id, 1, TRUE);
    $view_type = $this->getCookie('view-type', $this->mediaFolderFileSettings->get('default_view'));

    foreach ($tree as $term) {
      $classes = [
        'folder-icon',
        'folder-icon-folder-empty',
      ];

      if (!MediaFolderFileAccessCheck::hasFolderPermission('view', $term, $this->currentUser)) {
        continue;
      }
      $wrapper_classes = ['folder-folder'];
      if (!MediaFolderFileAccessCheck::hasFolderPermission('create', $term, $this->currentUser)) {
        $wrapper_classes[] = 'not-droppable';
      }

      $file_count = $this->getFileCount($term->id());

      $folders[] = [
        '#theme' => 'media_folder_folder',
        '#title' => Unicode::truncate($term->getName(), 30, FALSE, TRUE),
        '#weight' => $term->changed->value,
        '#desc' => (!empty($term->getDescription())) ? Markup::create(strip_tags($term->getDescription())) : '',
        '#link' => Url::fromRoute('media_folder_management.folder', ['folder' => $term->id()]),
        '#classes' => implode(' ', $classes),
        '#wrapper_classes' => implode(' ', $wrapper_classes),
        '#attr' => Markup::create('data-count="' . $file_count . '"'),
        '#size' => ($view_type == 'list') ? $file_count . ' ' . $this->t('items') : '',
        '#ftype' => ($view_type == 'list') ? $this->t('folder') : '',
        '#actions' => Markup::create($this->getItemActions($term)),
        '#fid' => $term->id(),
        '#cache' => [
          'tags' => [
            'media_folder_item:' . $term->id(),
          ],
          'contexts' => [
            'user',
          ],
        ],
      ];
    }
    $folders_ordered = $this->itemsOrder($folders);
    $this->getFolderFiles($folder, $folders_ordered);
    $navbar = $this->geAllUserFoldersTree($folder_id);
    $actions = $this->getFolderActions($folder);
    $actions_raw = $this->getRawActions($actions);
    $subfolders_class = (!$folder || !MediaFolderFileAccessCheck::hasFolderPermission('create', $folder, $this->currentUser)) ? 'not-droppable' : '';

    $table = [
      '#theme' => 'media_folder_page',
      '#title' => self::getTitle($folder),
      '#up_button' => $this->getUpButton($folder),
      '#breadcrumb' => $this->getBreadcrumb($folder),
      '#actions' => $actions,
      '#actions_raw' => ($actions_raw) ? $actions_raw : NULL,
      '#buttons' => Markup::create($this->getButtons()),
      '#form' => Markup::create($this->getForm()),
      '#folders' => [
        '#theme' => 'media_folder_folders',
        '#folders' => $folders_ordered,
      ],
      '#navbar' => [
        '#theme' => 'media_folder_navbar',
        '#folders' => $navbar,
      ],
      '#class' => $view_type,
      '#subfolders_class' => $subfolders_class,
      '#fid' => $folder_id,
      '#attached' => [
        'library' => 'media_folder_management/folders',
      ],
      '#cache' => [
        'tags' => [
          'media_folder_page:' . $folder_id,
        ],
        'contexts' => [
          'user',
        ],
      ],
    ];

    return $table;
  }

  /**
   * Display ajax Media folder.
   */
  public function fileExplorerAjax($folder = NULL) {
    if ($folder && ($folder->bundle() !== 'media_folder')) {
      throw new NotFoundHttpException();
    }

    $folder_id = ($folder) ? $folder->id() : 0;
    $folders = [];
    $tree = $this->storageController->loadTree('media_folder', $folder_id, 1, TRUE);
    $view_type = $this->getCookie('view-type', $this->mediaFolderFileSettings->get('default_view'));

    foreach ($tree as $term) {
      $classes = [
        'folder-icon',
        'folder-icon-folder-empty',
      ];

      if (!MediaFolderFileAccessCheck::hasFolderPermission('view', $term, $this->currentUser)) {
        continue;
      }

      $file_count = $this->getFileCount($term->id());

      $folders[] = [
        '#theme' => 'media_folder_folder',
        '#title' => $term->getName(),
        '#weight' => $term->changed->value,
        '#desc' => (!empty($term->getDescription())) ? Markup::create(strip_tags($term->getDescription())) : '',
        '#link' => Url::fromRoute('media_folder_management.folder', ['folder' => $term->id()]),
        '#classes' => implode(' ', $classes),
        '#wrapper_classes' => 'folder-folder',
        '#attr' => Markup::create('data-count="' . $file_count . '"'),
        '#size' => ($view_type == 'list') ? $file_count . ' ' . $this->t('items') : '',
        '#ftype' => ($view_type == 'list') ? $this->t('folder') : '',
        '#actions' => Markup::create($this->getItemActions($term)),
        '#fid' => $term->id(),
        '#cache' => [
          'tags' => [
            'media_folder_item:' . $term->id(),
          ],
          'contexts' => [
            'user',
          ],
        ],
      ];
    }
    $folders_ordered = $this->itemsOrder($folders);
    $this->getFolderFiles($folder, $folders_ordered);

    $actions = $this->getFolderActions($folder);
    $actions_raw = $this->getRawActions($actions);

    $up = $this->getUpButton($folder);
    $folders = [
      '#theme' => 'media_folder_folders',
      '#folders' => $folders_ordered,
    ];
    $navbar = [
      '#theme' => 'media_folder_navbar',
      '#folders' => $this->geAllUserFoldersTree($folder_id),
    ];
    $subfolders_class = (!$folder || !MediaFolderFileAccessCheck::hasFolderPermission('create', $folder, $this->currentUser)) ? 'not-droppable' : '';

    $response = [
      'navbar' => $this->renderer->render($navbar),
      'folders' => $this->renderer->render($folders) . '<div class="droppable-message">' . $this->t('Drop your file here!') . '</div>',
      'breadcrumb' => $this->getBreadcrumb($folder),
      'actions' => $this->renderer->render($actions),
      'actions_raw' => ($actions_raw) ? '<span class="close"></span>' . $this->renderer->render($actions_raw) : '',
      'buttons' => $this->getButtons(),
      'up_button' => (!$folder) ? $up : $this->renderer->render($up),
      'title' => self::getTitle($folder),
      'subfolders_class' => $subfolders_class,
      'fid' => $folder_id,
    ];

    return new JsonResponse($response);
  }

  /**
   * Media folder Perms.
   */
  public function fileExplorerPerms($folder) {
    $roles = $values = $header = $rows = [];

    $rolesObj = $this->entityManager->getStorage('user_role')->loadMultiple();
    foreach ($rolesObj as $roleObj) {
      $roles[] = $roleObj->id();
    }

    $fields = [
      'field_perms_create' => $this->t('Create'),
      'field_perms_edit_own' => $this->t('Edit Own'),
      'field_perms_edit_any' => $this->t('Edit Any'),
      'field_perms_view' => $this->t('View/Download'),
      'field_perms_lock' => $this->t('Lock'),
      'field_perms_delete_own' => $this->t('Delete Own'),
      'field_perms_delete' => $this->t('Delete'),
    ];

    $header['role'] = $this->t('Role');
    foreach ($fields as $field => $label) {
      $header[$field] = $label;

      if (!empty($folder->parent->entity) && $folder->field_perms_inherit->value) {
        $title = $this->t('Folder Permissions for "@name", inherited from parent', ['@name' => $folder->getName()]);
      }
      else {
        $title = $this->t('Folder Permissions for "@name"', ['@name' => $folder->getName()]);
      }

      $values[$field] = MediaFolderFileAccessCheck::getParentPermissionTarget($folder, $field);
    }

    foreach ($roles as $role) {
      $row = [];
      $row['role'] = $role;
      foreach ($values as $field => $field_values) {
        foreach ($field_values as $field_value) {
          if (!empty($field_value['target_id']) && $field_value['target_id'] == $role) {
            $row[$field] = 'X';
            break;
          }
          else {
            $row[$field] = '-';
          }
        }
      }
      $rows[] = $row;
    }

    return [
      '#theme' => 'table',
      '#caption' => $title,
      '#header' => $header,
      '#rows' => $rows,
      '#attributes' => ['id' => 'perms-table'],
      '#cache' => [
        'tags' => [
          'media_folder_page:' . $folder->id(),
        ],
        'contexts' => [
          'user',
        ],
      ],
      '#attached' => [
        'library' => ['media_folder_management/folders'],
      ],
    ];
  }

  /**
   * Media folder share link.
   */
  public function share($folder) {
    $url = Url::fromRoute('entity.taxonomy_term.canonical', ['taxonomy_term' => $folder->id()], ['absolute' => TRUE])->toString();
    return [
      '#theme' => 'media_folder_share',
      '#link' => $url,
      '#cache' => [
        'tags' => [
          'taxonomy_term:' . $folder->id(),
        ],
      ],
      '#attached' => [
        'library' => ['media_folder_management/folders'],
      ],
    ];
  }

  /**
   * Ajax upload files to folder.
   */
  public function ajaxUploadFile($folder) {
    $request = $this->requestStack->getCurrentRequest()->request->all();

    $response = [
      'messages' => [],
    ];

    if (!empty($request['request'])) {
      $field_config = $this->entityTypeManager()->getStorage('field_config')->load('media_folder_file.file.field_file');
      $field_settings = $field_config->getSettings();
      $validators = [];
      if (!empty($field_settings['file_extensions'])) {
        $validators['FileExtension'] = ['extensions' => $field_settings['file_extensions']];
      }
      if (!empty($field_settings['max_filesize'])) {
        $validators['FileSizeLimit'] = ['fileLimit' => $field_settings['max_filesize']];
      }

      foreach ($request['request'] as $post) {
        try {
          $files_data = preg_replace('#^data:application/\w+;base64,#i', '', $post['contents']);
          $files_data = preg_replace('#^data:image/\w+;base64,#i', '', $files_data);
          $file_data = base64_decode($files_data);
          $file = $this->fileRepository->writeData($file_data, 'temporary://' . $post['name'], FileExists::Rename);
          $file->status->value = 0;
          $file->save();

          $invalid = $this->fileValidator->validate($file, $validators);
          if ($invalid->count() > 0) {
            for ($i = 0; $i < $invalid->count(); $i++) {
              $response['messages'][] = [
                'type' => 'error',
                'message' => $invalid->get($i)->getMessage()->render(),
              ];
            }
          }
          else {
            $destination = $this->token->replace($field_settings['uri_scheme'] . '://' . $field_settings['file_directory']);
            $this->fileSystem->prepareDirectory($destination, FileSystemInterface::CREATE_DIRECTORY);
            $file->status->value = 1;
            $file->save();
            $this->fileRepository->move($file, $destination, FileExists::Rename);

            $folder_file = $this->entityTypeManager()->getStorage('media_folder_file')->create(
              [
                'bundle' => 'file',
                'name' => $post['name'],
                'field_file' => [
                  'target_id' => $file->id(),
                ],
                'field_folder' => [
                  'target_id' => $folder->id(),
                ],
                'status' => 1,
              ]
            );
            $folder_file->save();
            $this->fileUsage->add($file, 'media_folders_management', 'media_folder_file', $folder_file->id());

            $response['messages'][] = [
              'type' => 'status',
              'message' => $this->t('File @filename uploaded!', ['@filename' => $post['name']]),
            ];
          }
        }
        catch (\Exception $e) {
          $response['messages'][] = [
            'type' => 'error',
            'message' => $e->getMessage(),
          ];
        }
      }
    }

    return new JsonResponse($response);
  }

  /**
   * Ajax upload files to folder.
   */
  public function ajaxMoveInto($folder) {
    $request = $this->requestStack->getCurrentRequest()->request->all();

    $response = [
      'messages' => [],
    ];

    if (!empty($request['type']) && !empty($request['mid'])) {
      if ($request['type'] == 'folder') {
        $move = $this->entityTypeManager()->getStorage('taxonomy_term')->load($request['mid']);
        if ($move) {
          $move->parent->target_id = $folder->id();
          $move->save();
          $response['messages'][] = [
            'type' => 'status',
            'message' => $this->t('Folder @folder moved!', ['@folder' => $move->getName()]),
          ];
        }
      }
      elseif ($request['type'] == 'file') {
        $move = $this->entityTypeManager()->getStorage('media_folder_file')->load($request['mid']);
        if ($move) {
          $move->field_folder->target_id = $folder->id();
          $move->save();
          $response['messages'][] = [
            'type' => 'status',
            'message' => $this->t('File @filename moved!', ['@filename' => $move->getName()]),
          ];
        }
      }
    }

    return new JsonResponse($response);
  }

}
