<?php

namespace Drupal\media_folder_management\Entity\Controller;

use Drupal\Core\Link;
use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Url;
use Drupal\media_folder_management\MediaFolderFileInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Returns responses for media folder file routes.
 */
class MediaFolderFileController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public function __construct(DateFormatterInterface $date_formatter, RendererInterface $renderer, EntityTypeManagerInterface $entity_type_manager) {
    $this->dateFormatter = $date_formatter;
    $this->renderer = $renderer;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('date.formatter'),
      $container->get('renderer'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * Page title callback for a media folder file revision.
   *
   * @param \Drupal\media_folder_management\MediaFolderFileInterface $media_folder_file
   *   The media folder file ID.
   * @param \Drupal\media_folder_management\MediaFolderFileInterface $media_folder_file_revision
   *   The media folder file revision ID.
   *
   * @return string
   *   The page title.
   */
  public function revisionPageTitle(MediaFolderFileInterface $media_folder_file, MediaFolderFileInterface $media_folder_file_revision) {
    return $this->t('Revision of %title from %date', [
      '%title' => $media_folder_file_revision->label(),
      '%date' => $this->dateFormatter->format($media_folder_file_revision->getRevisionCreationTime()),
    ]);
  }

  /**
   * Generates an overview table of older revisions of a media folder file.
   *
   * @param \Drupal\media_folder_management\MediaFolderFileInterface $media_folder_file
   *   A media folder file object.
   *
   * @return array
   *   An array as expected by \Drupal\Core\Render\RendererInterface::render().
   */
  public function revisionOverview(MediaFolderFileInterface $media_folder_file) {
    $account = $this->currentUser();
    $langcode = $media_folder_file->language()->getId();
    $language_name = $media_folder_file->language()->getName();
    $languages = $media_folder_file->getTranslationLanguages();
    $has_translations = (count($languages) > 1);
    $media_folder_file_storage = $this->entityTypeManager->getStorage('media_folder_file');

    $build['#title'] = $has_translations ? $this->t('@language_name revisions for %title', [
      '@language_name' => $language_name,
      '%title' => $media_folder_file->label(),
    ]) : $this->t('Revisions for %title', ['%title' => $media_folder_file->label()]);
    $header = [$this->t('Revision'), $this->t('Operations')];

    $revert_permission = (((($account->hasPermission('administer own media folder files revisions') && ($account->id() == $media_folder_file->uid->entity->id()))) || $account->hasPermission('administer media folder files') || $account->hasPermission('administer all media folder files revisions'))  && $media_folder_file->access('update'));
    $delete_permission = (((($account->hasPermission('administer own media folder files revisions') && ($account->id() == $media_folder_file->uid->entity->id()))) || $account->hasPermission('administer media folder files') || $account->hasPermission('administer all media folder files revisions'))  && $media_folder_file->access('delete'));

    $rows = [];
    $default_revision = $media_folder_file->getRevisionId();
    $current_revision_displayed = FALSE;

    foreach ($this->getRevisionIds($media_folder_file, $media_folder_file_storage) as $vid) {
      $revision = $media_folder_file_storage->loadRevision($vid);
      if ($revision->hasTranslation($langcode) && $revision->getTranslation($langcode)->isRevisionTranslationAffected()) {
        $username = [
          '#theme' => 'username',
          '#account' => $revision->getRevisionUser(),
        ];
        $date = $this->dateFormatter->format($revision->getRevisionCreationTime(), 'short');
        $is_current_revision = $vid == $default_revision || (!$current_revision_displayed && $revision->wasDefaultRevision());
        if (!$is_current_revision) {
          $link = Link::fromTextAndUrl($date, new Url('entity.media_folder_file.revision', [
            'media_folder_file' => $media_folder_file->id(),
            'media_folder_file_revision' => $vid,
          ]))->toString();
        }
        else {
          $link = $media_folder_file->toLink($date)->toString();
          $current_revision_displayed = TRUE;
        }

        $row = [];
        $column = [
          'data' => [
            '#type' => 'inline_template',
            '#template' => '{% trans %}{{ date }} by {{ username }}{% endtrans %}{% if message %}<p class="revision-log">{{ message }}</p>{% endif %}',
            '#context' => [
              'date' => $link,
              'username' => $this->renderer->renderInIsolation($username),
              'message' => [
                '#markup' => $revision->getRevisionLogMessage(),
                '#allowed_tags' => Xss::getHtmlTagList(),
              ],
            ],
          ],
        ];
        $this->renderer->addCacheableDependency($column['data'], $username);
        $row[] = $column;

        if ($is_current_revision) {
          $row[] = [
            'data' => [
              '#prefix' => '<em>',
              '#markup' => $this->t('Current revision'),
              '#suffix' => '</em>',
            ],
          ];

          $rows[] = [
            'data' => $row,
            'class' => ['revision-current'],
          ];
        }
        else {
          $links = [];
          if ($revert_permission) {
            $links['revert'] = [
              'title' => $vid < $media_folder_file->getRevisionId() ? $this->t('Revert') : $this->t('Set as current revision'),
              'url' => Url::fromRoute('media_folder_file.revision_revert_confirm', [
                'media_folder_file' => $media_folder_file->id(),
                'media_folder_file_revision' => $vid,
              ]),
            ];
          }

          if ($delete_permission) {
            $links['delete'] = [
              'title' => $this->t('Delete'),
              'url' => Url::fromRoute('media_folder_file.revision_delete_confirm', [
                'media_folder_file' => $media_folder_file->id(),
                'media_folder_file_revision' => $vid,
              ]),
            ];
          }

          $row[] = [
            'data' => [
              '#type' => 'operations',
              '#links' => $links,
            ],
          ];

          $rows[] = $row;
        }
      }
    }

    $build['media_folder_file_revisions_table'] = [
      '#theme' => 'table',
      '#rows' => $rows,
      '#header' => $header,
      '#attributes' => ['class' => 'media-folder-file-revision-table'],
    ];

    $build['pager'] = ['#type' => 'pager'];

    return $build;
  }

  /**
   * Gets a list of items revision IDs for a specific $media_folder_file.
   *
   * @param \Drupal\media_folder_management\MediaFolderFileInterface $media_folder_file
   *   The media folder file entity.
   * @param mixed $media_folder_file_storage
   *   The media folder file storage handler.
   *
   * @return int[]
   *   Media folder file revision IDs (in descending order).
   */
  protected function getRevisionIds(MediaFolderFileInterface $media_folder_file, $media_folder_file_storage) {
    $result = $media_folder_file_storage->getQuery()
      ->allRevisions()
      ->condition($media_folder_file->getEntityType()->getKey('id'), $media_folder_file->id())
      ->sort($media_folder_file->getEntityType()->getKey('revision'), 'DESC')
      ->pager(50)
      ->accessCheck(FALSE)->execute();
    return array_keys($result);
  }

}
