<?php

namespace Drupal\media_folder_management\Entity;

use Drupal\Core\Entity\EditorialContentEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\media_folder_management\MediaFolderFileInterface;
use Drupal\user\UserInterface;

/**
 * Defines the Media folder file entity class.
 *
 * @ContentEntityType(
 *   id = "media_folder_file",
 *   label = @Translation("Media folder file"),
 *   label_singular = @Translation("Media folder file item"),
 *   label_plural = @Translation("Media folder file items"),
 *   label_count = @PluralTranslation(
 *     singular = "@count media folder file item",
 *     plural = "@count media folder file items"
 *   ),
 *   bundle_label = @Translation("Media folder file type"),
 *   handlers = {
 *     "storage" = "Drupal\media_folder_management\Entity\Storage\MediaFolderFileStorage",
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\media_folder_management\Entity\ListBuilder\MediaFolderFileListBuilder",
 *     "access" = "Drupal\media_folder_management\Entity\Access\MediaFolderFileAccessControlHandler",
 *     "form" = {
 *       "default" = "Drupal\media_folder_management\Entity\Form\MediaFolderFileForm",
 *       "add" = "Drupal\media_folder_management\Entity\Form\MediaFolderFileForm",
 *       "edit" = "Drupal\media_folder_management\Entity\Form\MediaFolderFileForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *       "delete-multiple-confirm" = "Drupal\Core\Entity\Form\DeleteMultipleForm",
 *       "lock" = "Drupal\media_folder_management\Entity\Form\MediaFolderFileLockForm",
 *     },
 *     "views_data" = "Drupal\media_folder_management\Entity\ViewsData\MediaFolderFileViewsData",
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     }
 *   },
 *   base_table = "media_folder_file",
 *   data_table = "media_folder_file_field_data",
 *   revision_table = "media_folder_file_revision",
 *   revision_data_table = "media_folder_file_field_revision",
 *   show_revision_ui = TRUE,
 *   entity_keys = {
 *     "id" = "lid",
 *     "revision" = "vid",
 *     "bundle" = "bundle",
 *     "label" = "name",
 *     "uuid" = "uuid",
 *     "published" = "status",
 *   },
 *   revision_metadata_keys = {
 *     "revision_user" = "revision_user",
 *     "revision_created" = "revision_created",
 *     "revision_log_message" = "revision_log_message",
 *   },
 *   bundle_entity_type = "media_folder_file_type",
 *   permission_granularity = "bundle",
 *   admin_permission = "administer media folder files",
 *   field_ui_base_route = "entity.media_folder_file_type.edit_form",
 *   common_reference_target = TRUE,
 *   links = {
 *     "add-page" = "/admin/content/media-folder-file/add",
 *     "add-form" = "/admin/content/media-folder-file/add/{media_folder_file_type}/{folder}",
 *     "canonical" = "/admin/content/media-folder-file/{media_folder_file}",
 *     "collection" = "/admin/content/media-folder-file",
 *     "delete-form" = "/admin/content/media-folder-file/{media_folder_file}/delete",
 *     "delete-multiple-form" = "/admin/content/media-folder-file/delete",
 *     "edit-form" = "/admin/content/media-folder-file/{media_folder_file}/edit",
 *     "revision" = "/admin/content/media-folder-file/{media_folder_file}/revisions/{media_folder_file_revision}/view",
 *   }
 * )
 */
class MediaFolderFile extends EditorialContentEntityBase implements MediaFolderFileInterface {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function getName() {
    $name = $this->getEntityKey('label');

    return $name;
  }

  /**
   * {@inheritdoc}
   */
  public function label() {
    return $this->getName();
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    return $this->set('name', $name);
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    return $this->set('created', $timestamp);
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('uid')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    return $this->set('uid', $account->id());
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('uid')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    return $this->set('uid', $uid);
  }

  /**
   * {@inheritdoc}
   */
  public function preSaveRevision(EntityStorageInterface $storage, \stdClass $record) {
    parent::preSaveRevision($storage, $record);

    $is_new_revision = $this->isNewRevision();
    if (!$is_new_revision && isset($this->original) && empty($record->revision_log_message)) {
      $record->revision_log_message = $this->original->revision_log_message->value;
    }

    if ($is_new_revision) {
      $record->revision_created = self::getRequestTime();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function prepareSave() {

    if (!isset($this->original) && $id = $this->id()) {
      $this->original = $this->entityTypeManager()
        ->getStorage('media_folder_file')
        ->loadUnchanged($id);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setRequired(TRUE)
      ->setTranslatable(TRUE)
      ->setRevisionable(TRUE)
      ->setDefaultValue('')
      ->setSetting('max_length', 255)
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('The user ID of the author.'))
      ->setRevisionable(TRUE)
      ->setDefaultValueCallback(static::class . '::getCurrentUserId')
      ->setSetting('target_type', 'user')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['status']->setRequired(TRUE)
      ->setSettings([
        'allowed_values' => [
          'active' => t('Active'),
          'locked' => t('Locked'),
        ],
      ])
      ->setDefaultValue('active')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -2,
      ])
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => -2,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Authored on'))
      ->setDescription(t('The time the media folder file item was created.'))
      ->setTranslatable(TRUE)
      ->setRevisionable(TRUE)
      ->setDefaultValueCallback(static::class . '::getRequestTime')
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => 10,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'timestamp',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time the media folder file item was last edited.'))
      ->setTranslatable(TRUE)
      ->setRevisionable(TRUE);

    return $fields;
  }

  /**
   * Default value callback for 'uid' base field definition.
   *
   * @see ::baseFieldDefinitions()
   *
   * @return int[]
   *   An array of default values.
   */
  public static function getCurrentUserId() {
    return [\Drupal::currentUser()->id()];
  }

  /**
   * {@inheritdoc}
   */
  public static function getRequestTime() {
    return \Drupal::time()->getRequestTime();
  }

  /**
   * {@inheritdoc}
   */
  public function getType() {
    return $this->bundle();
  }

}
