<?php

namespace Drupal\media_folder_management\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Entity\Term;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Session\AccountInterface;

/**
 * Form builder.
 */
class AddFolderForm extends FormBase {

  /**
   * The Get EntityTypeManagerInterface.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, AccountInterface $current_user) {
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $this->entityTypeManager->getStorage('user')->load($current_user->id());
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('entity_type.manager'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'add-folder-form';
  }

  /**
   * {@inheritdoc}
   */
  public function getFields() {
    $fields = [
      'field_perms_view' => $this->t('View/Download'),
      'field_perms_create' => $this->t('Create'),
      'field_perms_edit_any' => $this->t('Edit Any'),
      'field_perms_edit_own' => $this->t('Edit Own'),
      'field_perms_delete' => $this->t('Delete Any'),
      'field_perms_delete_own' => $this->t('Delete Own'),
      'field_perms_lock' => $this->t('Lock'),
    ];

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, mixed $folder = NULL) {
    $form['name'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Name'),
      '#default_value' => '',
      '#maxlength' => 40,
    ];

    $form['description'] = [
      '#type' => 'textarea',
      '#required' => FALSE,
      '#title' => $this->t('Description'),
      '#default_value' => '',
    ];

    if ($this->currentUser->hasPermission('administer folders') || $this->currentUser->hasPermission('administer media folder files')) {
      $roles = $this->entityTypeManager->getStorage('user_role')->loadMultiple();
      $fields = $this->getFields();

      $form['inherit'] = [
        '#title' => $this->t('Inherit Permissions'),
        '#type' => 'checkbox',
        '#default_value' => 1,
      ];
      if (!$folder) {
        $form['inherit']['#default_value'] = $form['inherit']['#value'] = 0;
        $form['inherit']['#disabled'] = TRUE;
      }

      $form['perms_wrapper'] = [
        '#type' => 'details',
        '#open' => TRUE,
        '#title' => $this->t('Permissions'),
        "#states" => [
          'visible' => [
            ':input[name="inherit"]' => ['checked' => FALSE],
          ],
        ],
      ];

      $form['perms_wrapper']['permissions'] = [
        '#type' => 'table',
        '#empty' => $this->t('No permissions found.'),
        '#header' => [$this->t('Role')],
        '#tree' => TRUE,
        '#title' => $this->t('Permissions'),
      ];

      foreach ($fields as $field => $name) {
        $form['perms_wrapper']['permissions']['#header'][] = [
          'data' => $name,
          'class' => ['checkbox'],
        ];
        foreach ($roles as $role) {
          $form['perms_wrapper']['permissions'][$role->id()]['label'] = [
            '#markup' => $role->label(),
          ];
          $form['perms_wrapper']['permissions'][$role->id()][$field] = [
            '#title' => $role->label(),
            '#title_display' => 'invisible',
            '#wrapper_attributes' => [
              'class' => ['checkbox'],
            ],
            '#type' => 'checkbox',
            '#default_value' => 0,
            '#parents' => [$field, $role->id()],
          ];
        }
      }
    }

    $form['parent'] = [
      '#type' => 'hidden',
      '#value' => ($folder) ? $folder->id() : 0,
    ];

    $form['actions'] = [
      '#type' => 'actions',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $term_values = [
      'parent' => [$values['parent']],
      'name' => $values['name'],
      'description' => $values['description'],
      'vid' => 'media_folder',
      'field_perms_inherit' => [
        'value' => $values['inherit'] ?? 1,
      ],
      'field_media_folder_uid' => [
        'target_id' => $this->currentUser->id(),
      ],
    ];

    if (isset($values['inherit']) && $values['inherit'] == FALSE) {
      $fields = $this->getFields();
      foreach ($fields as $key => $value) {
        if (!empty($values[$key])) {
          foreach ($values[$key] as $role => $allowed) {
            if ($allowed) {
              $term_values[$key][] = ['target_id' => $role];
            }
          }
        }
      }
    }

    $term = Term::create($term_values)->save();
    if ($term) {
      $this->messenger()->addStatus('Folder saved');
    }
  }

}
