<?php

namespace Drupal\media_folder_management\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form builder.
 */
class DeleteFolderForm extends ConfirmFormBase {

  /**
   * The Get EntityTypeManagerInterface.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Folder.
   *
   * @var \Drupal\taxonomy\Entity\TaxonomyTerm
   */
  protected $folder;

  /**
   * Sub folders.
   *
   * @var array
   */
  protected $subFolders;

  /**
   * Files.
   *
   * @var array
   */
  protected $files;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFolderChildren(&$children, &$files, $parent) {
    $files = $this->entityTypeManager->getStorage('media_folder_file')->loadByProperties([
      'field_folder' => $parent,
    ]);
    if (!empty($files)) {
      foreach ($files as $file) {
        $files[$file->id()] = $file->label();
      }
    }

    $folders = $this->entityTypeManager->getStorage('taxonomy_term')->loadByProperties([
      'parent' => $parent,
    ]);
    if (!empty($folders)) {
      foreach ($folders as $child) {
        $children[$child->id()] = $child->getName();
        $this->getFolderChildren($children, $files, $child->id());
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $folder = NULL) {
    $this->folder = $folder;
    $this->subFolders = [];
    $this->files = [];

    $this->getFolderChildren($this->subFolders, $this->files, $this->folder->id());
    if (!empty($this->subFolders)) {
      $form['children'] = [
        '#title' => $this->t('The following sub folders will also be deleted'),
        '#theme' => 'item_list',
        '#items' => $this->subFolders,
      ];
    }
    if (!empty($this->files)) {
      $form['files'] = [
        '#title' => $this->t('The following files will also be deleted'),
        '#theme' => 'item_list',
        '#items' => $this->files,
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    if ($this->folder->parent->target_id) {
      $form_state->setRedirect('media_folder_management.folder', [
        'folder' => $this->folder->parent->target_id,
      ]);
    }
    else {
      $form_state->setRedirect('media_folder_management.file_explorer');
    }

    foreach (array_keys($this->files) as $file_id) {
      $file = $this->entityTypeManager->getStorage('media_folder_file')->load($file_id);
      $file->delete();
    }
    foreach (array_keys($this->subFolders) as $folder_id) {
      $sub_folder = $this->entityTypeManager->getStorage('taxonomy_term')->load($folder_id);
      $sub_folder->delete();
    }
    $this->folder->delete();
    $this->messenger()->addStatus('Folder deleted');
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() : string {
    return "delete_folder_form";
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('media_folder_management.folder', [
      'folder' => $this->folder->id(),
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Do you want to delete folder @name?', ['@name' => $this->folder->getName()]);
  }

}
